@@ -8,6 +8,8 @@ namespace  {
   }
 
   bool isDark(const QColor& c) { return !isLight(c); }
+
+  constexpr int defaultIconLabelSize = 32;
 }
 
 // -------------------------------------------------------------------------------------------------
@@ -26,3 +28,20 @@ IconButton::IconButton(Font::Icon symbol, QWidget* parent)
                                                    : QColor(Qt::lightGray).lighter());
   setPalette(p);
 }
+
+// -------------------------------------------------------------------------------------------------
+IconLabel::IconLabel(Font::Icon symbol, QWidget* parent)
+  : QLabel(QChar(symbol), parent)
+{
+  QFont iconFont("projecteur-icons");
+  iconFont.setPixelSize(defaultIconLabelSize);
+  setFont(iconFont);
+}
+
+// -------------------------------------------------------------------------------------------------
+void IconLabel::setPixelSize(int pixelSize)
+{
+  auto font = this->font();
+  font.setPixelSize(pixelSize);
+  setFont(font);
+}
