@@ -1,6 +1,6 @@
 // This file is part of Projecteur - https://github.com/jahnf/projecteur - See LICENSE.md and README.md
 
-// Virtal Device to emit customized events from Projecteur device
+// Virtual Device to emit customized events from Projecteur device
 // The spotlight.cc grabs mouse inputs from Logitech Spotlight device.
 // This module is used when the input events are supposed to be forwarded to the system.
 
@@ -28,8 +28,6 @@ public:
   explicit VirtualDevice(Token, int fd);
   ~VirtualDevice();
 
-  void emitEvent(uint16_t type, uint16_t code, int val);
-  void emitEvent(struct input_event ie);
   void emitEvents(const struct input_event[], size_t num);
   void emitEvents(const std::vector<struct input_event>& events);
 };
